@@ -12,7 +12,6 @@ from loky import get_reusable_executor
 
 
 def func_async(i):
-    import os
     pid = os.getpid()
     return (2 * i, pid)
 
@@ -48,4 +47,4 @@ print("All the jobs were run in a process different from main process")
 
 assert len(set(pids)) == 1
 print("All the computation where run in a single `ProcessPoolExecutor` with "
-      "worker pid={}".format(pids[0]))
+      f"worker pid={pids[0]}")
