@@ -8,71 +8,28 @@
 #  * Add CustomizableLokyPickler to allow customizing pickling process
 #    on the fly.
 #
+import copyreg
 import io
-import os
-import sys
 import functools
-from multiprocessing import util
 import types
-try:
-    # Python 2 compat
-    from cPickle import loads as pickle_loads
-except ImportError:
-    from pickle import loads as pickle_loads
-    import copyreg
-
-from pickle import HIGHEST_PROTOCOL
-
-if sys.platform == "win32":
-    if sys.version_info[:2] > (3, 3):
-        from multiprocessing.reduction import duplicate
-    else:
-        from multiprocessing.forking import duplicate
+import sys
+import os
 
+from multiprocessing import util
+from pickle import loads, HIGHEST_PROTOCOL
 
 ###############################################################################
 # Enable custom pickling in Loky.
-# To allow instance customization of the pickling process, we use 2 classes.
-# _ReducerRegistry gives module level customization and CustomizablePickler
-# permits to use instance base custom reducers. Only CustomizablePickler
-# should be used.
 
-class _ReducerRegistry(object):
-    """Registry for custom reducers.
+_dispatch_table = {}
 
-    HIGHEST_PROTOCOL is selected by default as this pickler is used
-    to pickle ephemeral datastructures for interprocess communication
-    hence no backward compatibility is required.
-
-    """
-
-    # We override the pure Python pickler as its the only way to be able to
-    # customize the dispatch table without side effects in Python 2.6
-    # to 3.2. For Python 3.3+ leverage the new dispatch_table
-    # feature from http://bugs.python.org/issue14166 that makes it possible
-    # to use the C implementation of the Pickler which is faster.
-
-    dispatch_table = {}
-
-    @classmethod
-    def register(cls, type, reduce_func):
-        """Attach a reducer function to a given type in the dispatch table."""
-        if sys.version_info < (3,):
-            # Python 2 pickler dispatching is not explicitly customizable.
-            # Let us use a closure to workaround this limitation.
-            def dispatcher(cls, obj):
-                reduced = reduce_func(obj)
-                cls.save_reduce(obj=obj, *reduced)
-            cls.dispatch_table[type] = dispatcher
-        else:
-            cls.dispatch_table[type] = reduce_func
 
+def register(type_, reduce_function):
+    _dispatch_table[type_] = reduce_function
 
 ###############################################################################
 # Registers extra pickling routines to improve picklization  for loky
 
-register = _ReducerRegistry.register
-
 
 # make methods picklable
 def _reduce_method(m):
@@ -157,15 +114,16 @@ def set_loky_pickler(loky_pickler=None):
             loky_pickler_cls = module_pickle.Pickler
         except (ImportError, AttributeError) as e:
             extra_info = ("\nThis error occurred while setting loky_pickler to"
-                          " '{}', as required by the env variable LOKY_PICKLER"
-                          " or the function set_loky_pickler."
-                          .format(loky_pickler))
+                          f" '{loky_pickler}', as required by the env variable "
+                          "LOKY_PICKLER or the function set_loky_pickler.")
             e.args = (e.args[0] + extra_info,) + e.args[1:]
             e.msg = e.args[0]
             raise e
 
-    util.debug("Using '{}' for serialization."
-               .format(loky_pickler if loky_pickler else "cloudpickle"))
+    util.debug(
+        f"Using '{loky_pickler if loky_pickler else 'cloudpickle'}' for "
+        "serialization."
+    )
 
     class CustomizablePickler(loky_pickler_cls):
         _loky_pickler_cls = loky_pickler_cls
@@ -195,43 +153,32 @@ def set_loky_pickler(loky_pickler=None):
             loky_pickler_cls.__init__(self, writer, protocol=protocol)
             if reducers is None:
                 reducers = {}
-            if sys.version_info < (3,):
-                self.dispatch = loky_pickler_cls.dispatch.copy()
-                self.dispatch.update(_ReducerRegistry.dispatch_table)
+
+            if hasattr(self, "dispatch_table"):
+                # Force a copy that we will update without mutating the
+                # any class level defined dispatch_table.
+                loky_dt = dict(self.dispatch_table)
             else:
-                if hasattr(self, "dispatch_table"):
-                    # Force a copy that we will update without mutating the
-                    # any class level defined dispatch_table.
-                    loky_dt = dict(self.dispatch_table)
-                else:
-                    # Use standard reducers as bases
-                    loky_dt = copyreg.dispatch_table.copy()
+                # Use standard reducers as bases
+                loky_dt = copyreg.dispatch_table.copy()
 
-                # Register loky specific reducers
-                loky_dt.update(_ReducerRegistry.dispatch_table)
+            # Register loky specific reducers
+            loky_dt.update(_dispatch_table)
 
-                # Set the new dispatch table, taking care of the fact that we
-                # need to use the member_descriptor when we inherit from a
-                # subclass of the C implementation of the Pickler base class
-                # with an class level dispatch_table attribute.
-                self._set_dispatch_table(loky_dt)
+            # Set the new dispatch table, taking care of the fact that we
+            # need to use the member_descriptor when we inherit from a
+            # subclass of the C implementation of the Pickler base class
+            # with an class level dispatch_table attribute.
+            self._set_dispatch_table(loky_dt)
 
-            # Register custom reducers
+            # Register the reducers
             for type, reduce_func in reducers.items():
                 self.register(type, reduce_func)
 
         def register(self, type, reduce_func):
             """Attach a reducer function to a given type in the dispatch table.
             """
-            if sys.version_info < (3,):
-                # Python 2 pickler dispatching is not explicitly customizable.
-                # Let us use a closure to workaround this limitation.
-                def dispatcher(self, obj):
-                    reduced = reduce_func(obj)
-                    self.save_reduce(obj=obj, *reduced)
-                self.dispatch[type] = dispatcher
-            else:
-                self.dispatch_table[type] = reduce_func
+            self.dispatch_table[type] = reduce_func
 
     _LokyPickler = CustomizablePickler
     _loky_pickler_name = loky_pickler
@@ -251,13 +198,6 @@ def get_loky_pickler():
 set_loky_pickler()
 
 
-def loads(buf):
-    # Compat for python2.7 version
-    if sys.version_info < (3, 3) and isinstance(buf, io.BytesIO):
-        buf = buf.getvalue()
-    return pickle_loads(buf)
-
-
 def dump(obj, file, reducers=None, protocol=None):
     '''Replacement for pickle.dump() using _LokyPickler.'''
     global _LokyPickler
@@ -269,12 +209,11 @@ def dumps(obj, reducers=None, protocol=None):
 
     buf = io.BytesIO()
     dump(obj, buf, reducers=reducers, protocol=protocol)
-    if sys.version_info < (3, 3):
-        return buf.getvalue()
     return buf.getbuffer()
 
 
 __all__ = ["dump", "dumps", "loads", "register", "set_loky_pickler"]
 
 if sys.platform == "win32":
+    from multiprocessing.reduction import duplicate
     __all__ += ["duplicate"]
