@@ -1,24 +1,23 @@
-from __future__ import print_function
-import os
 import sys
 import time
 import math
-import psutil
 import pytest
 import threading
+from time import sleep
 
-from loky._base import TimeoutError
+from loky import TimeoutError, get_reusable_executor
 from loky.backend import get_context
-from loky import get_reusable_executor, cpu_count
 
 
-# Compat Travis
-CPU_COUNT = cpu_count()
-if os.environ.get("TRAVIS_OS_NAME") is not None and sys.version_info < (3, 4):
-    # Hard code number of cpu in travis as cpu_count return 32 whereas we
-    # only access 2 cores.
-    # This is done automatically by cpu_count for Python >= 3.4
-    CPU_COUNT = 2
+try:
+    import psutil
+    psutil_exceptions = (psutil.NoSuchProcess, psutil.AccessDenied)
+except ImportError:
+    psutil = None
+    psutil_exceptions = ()
+
+
+IS_PYPY = hasattr(sys, "pypy_version_info")
 
 # Set a large timeout as it should only be reached in case of deadlocks
 TIMEOUT = 40
@@ -46,7 +45,7 @@ def _direct_children_with_cmdline(p):
                 # when the process is being terminated by the OS.
                 continue
             children_with_cmdline.append((c, cmdline))
-        except (OSError, psutil.NoSuchProcess, psutil.AccessDenied):
+        except (OSError,) + psutil_exceptions:
             # These errors indicate that the process has terminated while
             # we were processing the info. Just discard it.
             pass
@@ -56,23 +55,27 @@ def _direct_children_with_cmdline(p):
 def _running_children_with_cmdline(p):
     all_children = _direct_children_with_cmdline(p)
     workers = [(c, cmdline) for c, cmdline in all_children
-               if (u'semaphore_tracker' not in cmdline and
-                   u'resource_tracker' not in cmdline and
-                   u'multiprocessing.forkserver' not in cmdline)]
+               if ('semaphore_tracker' not in cmdline and
+                   'resource_tracker' not in cmdline and
+                   'multiprocessing.forkserver' not in cmdline)]
 
     forkservers = [c for c, cmdline in all_children
-                   if u'multiprocessing.forkserver' in cmdline]
+                   if 'multiprocessing.forkserver' in cmdline]
     for fs in forkservers:
-        workers.extend(_direct_children_with_cmdline(fs))
+        workers += _direct_children_with_cmdline(fs)
     return workers
 
 
 def _check_subprocesses_number(executor, expected_process_number=None,
                                expected_max_process_number=None, patience=100):
+    if not psutil:
+        # psutil is not installed, we cannot check the number of subprocesses
+        return
+
     # Wait for terminating processes to disappear
     children_cmdlines = _running_children_with_cmdline(psutil.Process())
     pids_cmdlines = [(c.pid, cmdline) for c, cmdline in children_cmdlines]
-    children_pids = set(pid for pid, _ in pids_cmdlines)
+    children_pids = {pid for pid, _ in pids_cmdlines}
     if executor is not None:
         worker_pids = set(executor._processes.keys())
     else:
@@ -92,7 +95,7 @@ def _check_subprocesses_number(executor, expected_process_number=None,
             # psutil even though it have been terminated. Wait for the system
             # clean up in this case.
             for _ in range(patience):
-                if len(_running_children_with_cmdline(psutil.Process())) == 0:
+                if not _running_children_with_cmdline(psutil.Process()):
                     break
                 time.sleep(.1)
             else:
@@ -120,18 +123,20 @@ def _check_executor_started(executor):
 
 
 class ExecutorMixin:
-    worker_count = 5
+    # PyPy is slower to create worker processes and creating
+    # too many processes can cause some tests to timeout.
+    worker_count = 2 if IS_PYPY else 5
 
     @classmethod
     def setup_class(cls):
-        print("setup class with {}".format(cls.context))
+        print(f"setup class with {cls.context}")
         global _test_event
         if _test_event is None:
             _test_event = cls.context.Event()
 
     @classmethod
     def teardown_class(cls):
-        print("teardown class with {}".format(cls.context))
+        print(f"teardown class with {cls.context}")
         global _test_event
         if _test_event is not None:
             _test_event = None
@@ -153,18 +158,41 @@ class ExecutorMixin:
         # Make sure executor is not broken if it should not be
         executor = getattr(self, 'executor', None)
         if executor is not None:
-            expect_broken_pool = hasattr(method, "broken_pool")  # old pytest
-            for mark in getattr(method, "pytestmark", []):
-                if mark.name == "broken_pool":
-                    expect_broken_pool = True
-            is_actually_broken = executor._flags.broken is not None
-            assert is_actually_broken == expect_broken_pool
+            try:
+                # old pytest markers:
+                expect_broken_pool = hasattr(method, "broken_pool")
+                # new pytest markers:
+                for mark in getattr(method, "pytestmark", []):
+                    if mark.name == "broken_pool":
+                        expect_broken_pool = True
 
-            t_start = time.time()
-            executor.shutdown(wait=True, kill_workers=True)
-            dt = time.time() - t_start
-            assert dt < 10, "Executor took too long to shutdown"
-        _check_subprocesses_number(executor, 0)
+                if expect_broken_pool:
+                    for _ in range(10):
+                        # The executor manager thread can take some time to
+                        # mark the executor broken.
+                        is_actually_broken = executor._flags.broken is not None
+                        if is_actually_broken:
+                            break
+                        sleep(0.1)
+                    else:
+                        raise AssertionError(
+                            "The executor was not flagged broken at the end of "
+                            f" {method.__qualname__} as expected."
+                        )
+                else:
+                    # Check that the executor is not broken right away to avoid
+                    # wasting CI time. False negative should be very rare.
+                    is_actually_broken = executor._flags.broken is not None
+                    assert not is_actually_broken
+            finally:
+                # Always shutdown the executor, with SIGKILL if the executor
+                # is actually broken.
+                kill_workers = executor._flags.broken is not None
+                t_start = time.time()
+                executor.shutdown(wait=True, kill_workers=kill_workers)
+                dt = time.time() - t_start
+                assert dt < 10, "Executor took too long to shutdown"
+                _check_subprocesses_number(executor, 0)
 
     def _prime_executor(self):
         # Make sure that the executor is ready to do work before running the
@@ -176,13 +204,16 @@ class ExecutorMixin:
 
     @classmethod
     def check_no_running_workers(cls, patience=5, sleep_duration=0.01):
+        if psutil is None:
+            return
+
         deadline = time.time() + patience
 
         while time.time() <= deadline:
             time.sleep(sleep_duration)
             p = psutil.Process()
             workers = _running_children_with_cmdline(p)
-            if len(workers) == 0:
+            if not workers:
                 return
 
         # Patience exhausted: log the remaining workers command line and
@@ -192,9 +223,9 @@ class ExecutorMixin:
             print(w.pid, w.status(), end='\n', file=sys.stderr)
             print(cmdline, end='\n\n', file=sys.stderr)
         raise AssertionError(
-            'Expected no more running worker processes but got %d after'
-            ' waiting %0.3fs.'
-            % (len(workers), patience))
+            f'Expected no more running worker processes but got {len(workers)}'
+            f' after waiting {patience:0.3f}s.'
+        )
 
 
 class ReusableExecutorMixin:
