@@ -3,8 +3,8 @@ import os
 import sys
 import shutil
 import tempfile
+import warnings
 from subprocess import check_output
-import subprocess
 
 import pytest
 
@@ -53,17 +53,18 @@ def test_cpu_count_affinity():
         pytest.skip()
 
     res = check_output([taskset_bin, '-c', '0',
-                        python_bin, '-c', cpu_count_cmd.format(args='')])
-    
+                        python_bin, '-c', cpu_count_cmd.format(args='')],
+                       text=True)
+
     res_physical = check_output([
         taskset_bin, '-c', '0', python_bin, '-c',
-        cpu_count_cmd.format(args='only_physical_cores=True')])
+        cpu_count_cmd.format(args='only_physical_cores=True')], text=True)
 
-    assert res.strip().decode('utf-8') == '1'
-    assert res_physical.strip().decode('utf-8') == '1'
+    assert res.strip() == '1'
+    assert res_physical.strip() == '1'
 
 
-def test_cpu_count_cfs_limit():
+def test_cpu_count_cgroup_limit():
     if sys.platform == "win32":
         pytest.skip()
 
@@ -72,19 +73,43 @@ def test_cpu_count_cfs_limit():
 
     docker_bin = shutil.which('docker')
     if docker_bin is None:
-        raise pytest.skip()
+        raise pytest.skip("docker is required to run this test")
 
-    loky_path = os.path.abspath(os.path.dirname(loky.__file__))
+    loky_module_path = os.path.abspath(os.path.dirname(loky.__file__))
+    loky_project_path = os.path.abspath(os.path.join(loky_module_path, os.pardir))
 
-    # The following will always run using the Python 3.6 docker image.
+    # The following will always run using the Python 3.7 docker image.
     # We mount the loky source as /loky inside the container,
     # so it can be imported when running commands under /
-    res = check_output([docker_bin, 'run', '--rm', '--cpus', '0.5',
-                        '-v', '%s:/loky' % loky_path,
-                        'python:3.6',
-                        'python', '-c', cpu_count_cmd.format(args='')])
 
-    assert res.strip().decode('utf-8') == '1'
+    # Tell docker to configure the Cgroup quota to use 0.5 CPU, loky will
+    # always detect 1 CPU because it rounds up to the next integer.
+    res_500_mCPU = int(check_output(
+        f"{docker_bin} run --rm --cpus 0.5 -v {loky_project_path}:/loky python:3.7 "
+        f"/bin/bash -c 'pip install --quiet -e /loky ; "
+        f"python -c \"{cpu_count_cmd.format(args='')}\"'",
+        shell=True, text=True
+    ).strip())
+    assert res_500_mCPU == 1
+
+    # Limiting to 1.5 CPUs can lead to 1 if there is only 1 CPU on the machine or
+    # 2 if there are 2 CPUs or more.
+    res_1500_mCPU = int(check_output(
+        f"{docker_bin} run --rm --cpus 1.5 -v {loky_project_path}:/loky python:3.7 "
+        f"/bin/bash -c 'pip install --quiet -e /loky ; "
+        f"python -c \"{cpu_count_cmd.format(args='')}\"'",
+        shell=True, text=True
+    ).strip())
+    assert res_1500_mCPU in (1, 2)
+
+    # By default there is no limit: use all available CPUs.
+    res_default = int(check_output(
+        f"{docker_bin} run --rm -v {loky_project_path}:/loky python:3.7 "
+        f"/bin/bash -c 'pip install --quiet -e /loky ; "
+        f"python -c \"{cpu_count_cmd.format(args='')}\"'",
+        shell=True, text=True
+    ).strip())
+    assert res_default >= res_1500_mCPU
 
 
 def test_only_physical_cores_error():
@@ -92,7 +117,7 @@ def test_only_physical_cores_error():
     # unable to retrieve the number of physical cores.
     if sys.platform != "linux":
         pytest.skip()
-    
+
     # if number of available cpus is already restricted, cpu_count will return
     # that value and no warning is issued even if only_physical_cores == True.
     # (tested in another test: test_only_physical_cores_with_user_limitation
@@ -100,11 +125,9 @@ def test_only_physical_cores_error():
     if _cpu_count_user(cpu_count_mp) < cpu_count_mp:
         pytest.skip()
 
-    start_dir = os.path.abspath('.')
-
     with tempfile.TemporaryDirectory() as tmp_dir:
         # Write bad lscpu program
-        lscpu_path = tmp_dir + '/lscpu'
+        lscpu_path = f'{tmp_dir}/lscpu'
         with open(lscpu_path, 'w') as f:
             f.write("#!/bin/sh\n"
                     "exit(1)")
@@ -112,7 +135,7 @@ def test_only_physical_cores_error():
 
         try:
             old_path = os.environ['PATH']
-            os.environ['PATH'] = tmp_dir + ":" + old_path
+            os.environ['PATH'] = f'{tmp_dir}:{old_path}'
 
             # clear the cache otherwise the warning is not triggered
             import loky.backend.context
@@ -123,9 +146,9 @@ def test_only_physical_cores_error():
                 cpu_count(only_physical_cores=True)
 
             # Should not warn the second time
-            with pytest.warns(None) as record:
+            with warnings.catch_warnings():
+                warnings.simplefilter('error')
                 cpu_count(only_physical_cores=True)
-                assert not record
 
         finally:
             os.environ['PATH'] = old_path
