@@ -2,14 +2,15 @@ import os
 import sys
 import gc
 import ctypes
-import psutil
 import pytest
 import warnings
 import threading
 from time import sleep
 from multiprocessing import util, current_process
 from pickle import PicklingError, UnpicklingError
-from distutils.version import LooseVersion
+
+import cloudpickle
+from packaging.version import Version
 
 import loky
 from loky import cpu_count
@@ -17,13 +18,17 @@ from loky import get_reusable_executor
 from loky.process_executor import _RemoteTraceback, TerminatedWorkerError
 from loky.process_executor import BrokenProcessPool, ShutdownExecutorError
 from loky.reusable_executor import _ReusablePoolExecutor
-import cloudpickle
+
+try:
+    import psutil
+except ImportError:
+    psutil = None
 
 from ._executor_mixin import ReusableExecutorMixin
 from .utils import TimingWrapper, id_sleep, check_python_subprocess_call
 from .utils import filter_match
 
-cloudpickle_version = LooseVersion(cloudpickle.__version__)
+cloudpickle_version = Version(cloudpickle.__version__)
 
 # Compat windows
 if sys.platform == "win32":
@@ -40,32 +45,26 @@ try:
 except ImportError:
     np = None
 
-# Backward compat for python2 cPickle module
-PICKLING_ERRORS = (PicklingError,)
-try:
-    import cPickle
-    PICKLING_ERRORS += (cPickle.PicklingError,)
-except ImportError:
-    pass
-
 
 def clean_warning_registry():
     """Safe way to reset warnings."""
     warnings.resetwarnings()
     reg = "__warningregistry__"
-    for mod_name, mod in list(sys.modules.items()):
+    for mod in list(sys.modules.values()):
         if hasattr(mod, reg):
             getattr(mod, reg).clear()
 
 
 def wait_dead(worker, n_tries=1000, delay=0.001):
     """Wait for process pid to die"""
-    for i in range(n_tries):
+    for _ in range(n_tries):
         if worker.exitcode is not None:
             return
         sleep(delay)
-    raise RuntimeError("Process %d failed to die for at least %0.3fs" %
-                       (worker.pid, delay * n_tries))
+    raise RuntimeError(
+        f"Process {worker.pid} failed to die for at least "
+        f"{delay * n_tries:0.3f}s"
+    )
 
 
 def crash():
@@ -103,7 +102,7 @@ def kill_friend(pid, delay=0):
         if psutil.pid_exists(pid):
             util.debug("Fail to kill an alive process?!?")
             raise e
-        util.debug("process {} was already dead".format(pid))
+        util.debug(f"process {pid} was already dead")
 
 
 def raise_error(etype=UnpicklingError, message=None):
@@ -132,43 +131,43 @@ def do_nothing(arg):
     return True
 
 
-class CrashAtPickle(object):
+class CrashAtPickle:
     """Bad object that triggers a segfault at pickling time."""
     def __reduce__(self):
         crash()
 
 
-class CrashAtUnpickle(object):
+class CrashAtUnpickle:
     """Bad object that triggers a segfault at unpickling time."""
     def __reduce__(self):
         return crash, ()
 
 
-class ExitAtPickle(object):
+class ExitAtPickle:
     """Bad object that triggers a segfault at pickling time."""
     def __reduce__(self):
         exit()
 
 
-class ExitAtUnpickle(object):
+class ExitAtUnpickle:
     """Bad object that triggers a process exit at unpickling time."""
     def __reduce__(self):
         return exit, ()
 
 
-class CExitAtPickle(object):
+class CExitAtPickle:
     """Bad object that triggers a segfault at pickling time."""
     def __reduce__(self):
         c_exit()
 
 
-class CExitAtUnpickle(object):
+class CExitAtUnpickle:
     """Bad object that triggers a process exit at unpickling time."""
     def __reduce__(self):
         return c_exit, ()
 
 
-class ErrorAtPickle(object):
+class ErrorAtPickle:
     """Bad object that raises an error at pickling time."""
     def __init__(self, fail=True):
         self.fail = fail
@@ -180,7 +179,7 @@ class ErrorAtPickle(object):
             return id, (42, )
 
 
-class ErrorAtUnpickle(object):
+class ErrorAtUnpickle:
     """Bad object that triggers a process exit at unpickling time."""
     def __init__(self, etype=UnpicklingError, message='the error message'):
         self.etype = etype
@@ -190,14 +189,14 @@ class ErrorAtUnpickle(object):
         return raise_error, (self.etype, self.message)
 
 
-class CrashAtGCInWorker(object):
+class CrashAtGCInWorker:
     """Bad object that triggers a segfault at call item GC time"""
     def __del__(self):
         if current_process().name != "MainProcess":
             crash()
 
 
-class CExitAtGCInWorker(object):
+class CExitAtGCInWorker:
     """Exit worker at call item GC time"""
     def __del__(self):
         if current_process().name != "MainProcess":
@@ -211,27 +210,27 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         (id, (ExitAtPickle(),), PicklingError, None),
         (id, (ErrorAtPickle(),), PicklingError, None),
         # Check problem occuring while unpickling a task on workers
-        (id, (ExitAtUnpickle(),), BrokenProcessPool, r"SystemExit"),
+        (id, (ExitAtUnpickle(),), BrokenProcessPool, "SystemExit"),
         (id, (CExitAtUnpickle(),), TerminatedWorkerError, r"EXIT\(0\)"),
-        (id, (ErrorAtUnpickle(),), BrokenProcessPool, r"UnpicklingError"),
-        (id, (CrashAtUnpickle(),), TerminatedWorkerError, r"SIGSEGV"),
+        (id, (ErrorAtUnpickle(),), BrokenProcessPool, "UnpicklingError"),
+        (id, (CrashAtUnpickle(),), TerminatedWorkerError, "SIGSEGV"),
         # Check problem occuring during function execution on workers
-        (crash, (), TerminatedWorkerError, r"SIGSEGV"),
+        (crash, (), TerminatedWorkerError, "SIGSEGV"),
         (exit, (), SystemExit, None),
         (c_exit, (), TerminatedWorkerError, r"EXIT\(0\)"),
         (raise_error, (RuntimeError,), RuntimeError, None),
         # Check problem occuring while pickling a task result
         # on workers
-        (return_instance, (CrashAtPickle,), TerminatedWorkerError, r"SIGSEGV"),
+        (return_instance, (CrashAtPickle,), TerminatedWorkerError, "SIGSEGV"),
         (return_instance, (ExitAtPickle,), SystemExit, None),
         (return_instance, (CExitAtPickle,), TerminatedWorkerError,
          r"EXIT\(0\)"),
         (return_instance, (ErrorAtPickle,), PicklingError, None),
         # Check problem occuring while unpickling a task in
         # the result_handler thread
-        (return_instance, (ExitAtUnpickle,), BrokenProcessPool, r"SystemExit"),
+        (return_instance, (ExitAtUnpickle,), BrokenProcessPool, "SystemExit"),
         (return_instance, (ErrorAtUnpickle,), BrokenProcessPool,
-         r"UnpicklingError"),
+         "UnpicklingError"),
     ]
 
     @pytest.mark.parametrize("func, args, expected_err, match", crash_cases)
@@ -334,7 +333,7 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         # in broken state:
         sleep(.5)
         with pytest.raises(TerminatedWorkerError,
-                           match=filter_match(r"SIGKILL")):
+                           match=filter_match("SIGKILL")):
             executor.submit(id_sleep, 42, 0.1).result()
 
         # the get_reusable_executor factory should be able to create a new
@@ -345,6 +344,7 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
     @pytest.mark.parametrize("n_proc", [1, 2, 5, 13])
     def test_crash_races(self, n_proc):
         """Test the race conditions in reusable_executor crash handling"""
+        pytest.importorskip("psutil")  # required for kill_friend & co
 
         if (sys.platform == 'win32' and sys.version_info >= (3, 8)
                 and n_proc > 5):
@@ -365,9 +365,9 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         res = executor.map(sleep_then_check_pids_exist,
                            [(.0001 * (j // 2), pids)
                             for j in range(2 * n_proc)])
-        assert all(list(res))
+        assert all(res)
         with pytest.raises(TerminatedWorkerError,
-                           match=filter_match(r"SIGKILL")):
+                           match=filter_match("SIGKILL")):
             res = executor.map(kill_friend, pids[::-1])
             list(res)
 
@@ -393,9 +393,9 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
     def test_queue_full_deadlock(self):
         executor = get_reusable_executor(max_workers=1)
         fs_fail = [executor.submit(do_nothing, ErrorAtPickle(True))
-                   for i in range(100)]
+                   for _ in range(100)]
         fs = [executor.submit(do_nothing, ErrorAtPickle(False))
-              for i in range(100)]
+              for _ in range(100)]
         with pytest.raises(PicklingError):
             fs_fail[99].result()
         assert fs[99].result()
@@ -443,6 +443,8 @@ class TestTerminateExecutor(ReusableExecutorMixin):
 
     def test_shutdown_deadlock(self):
         """Test recovery if killed after resize call"""
+        pytest.importorskip("psutil")  # required for kill_friend & co
+
         # Test the executor.shutdown call do not cause deadlock
         executor = get_reusable_executor(max_workers=2, timeout=None)
         executor.map(id, range(2))  # start the worker processes
@@ -467,7 +469,7 @@ class TestTerminateExecutor(ReusableExecutorMixin):
         assert f2.result() == 42
 
     @pytest.mark.parametrize("bad_object, match", [
-        (CrashAtGCInWorker, r"SIGSEGV"), (CExitAtGCInWorker, r"EXIT\(0\)")])
+        (CrashAtGCInWorker, "SIGSEGV"), (CExitAtGCInWorker, r"EXIT\(0\)")])
     def test_call_item_gc_crash_or_exit(self, bad_object, match):
         executor = get_reusable_executor(max_workers=1)
         bad_object = bad_object()
@@ -481,13 +483,15 @@ class TestTerminateExecutor(ReusableExecutorMixin):
         # when processing subsequently dispatched tasks:
         with pytest.raises(TerminatedWorkerError, match=filter_match(match)):
             executor.submit(gc.collect).result()
-            for r in executor.map(sleep, [.1] * 100):
+            for _ in executor.map(sleep, [.1] * 100):
                 pass
 
 
 class TestResizeExecutor(ReusableExecutorMixin):
     def test_reusable_executor_resize(self):
         """Test reusable_executor resizing"""
+        # required by sleep_then_check_pids_exist
+        pytest.importorskip("psutil")
 
         executor = get_reusable_executor(max_workers=2, timeout=None)
         executor.map(id, range(2))
@@ -512,7 +516,7 @@ class TestResizeExecutor(ReusableExecutorMixin):
 
         # Requesting the same number of process should not impact the executor
         # nor kill the processed
-        old_pid = next(iter((executor._processes.keys())))
+        old_pid = next(iter(executor._processes.keys()))
         unchanged_executor = get_reusable_executor(max_workers=1, timeout=None)
         assert len(unchanged_executor._processes) == 1
         assert unchanged_executor is executor
@@ -545,6 +549,8 @@ class TestResizeExecutor(ReusableExecutorMixin):
 
     def test_kill_after_resize_call(self):
         """Test recovery if killed after resize call"""
+        pytest.importorskip("psutil")  # required for kill_friend & co
+
         # Test the executor resizing called before a kill arrive
         executor = get_reusable_executor(max_workers=2, timeout=None)
         executor.map(id, range(2))  # trigger the creation of worker processes
@@ -594,8 +600,6 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
             executor._resize(max_workers=None)
 
     @pytest.mark.skipif(sys.platform == "win32", reason="No fork on windows")
-    @pytest.mark.skipif(sys.version_info <= (3, 4),
-                        reason="No context before 3.4")
     def test_invalid_context(self):
         """Raise error on invalid context"""
 
@@ -622,7 +626,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
             e.submit(id, 42).result()
             print("ok")
         """
-        check_python_subprocess_call(code, stdout_regex=r"ok")
+        check_python_subprocess_call(code, stdout_regex="ok")
 
     def test_reused_flag(self):
         executor, _ = _ReusablePoolExecutor.get_reusable_executor(
@@ -638,8 +642,8 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         )
         assert not reused
 
-    @pytest.mark.xfail(cloudpickle_version >= LooseVersion("0.5.4") and
-                       cloudpickle_version <= LooseVersion("0.7.0"),
+    @pytest.mark.xfail(cloudpickle_version >= Version("0.5.4") and
+                       cloudpickle_version <= Version("0.7.0"),
                        reason="Known issue in cloudpickle")
     # https://github.com/cloudpipe/cloudpickle/pull/240
     def test_interactively_defined_nested_functions(self):
@@ -674,7 +678,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
 
             print("ok")
         """
-        check_python_subprocess_call(code, stdout_regex=r"ok")
+        check_python_subprocess_call(code, stdout_regex="ok")
 
     def test_interactively_defined_recursive_functions(self):
         # Check that it's possible to call a recursive function defined
@@ -705,7 +709,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
 
             print("ok")
         """
-        check_python_subprocess_call(code, stdout_regex=r"ok")
+        check_python_subprocess_call(code, stdout_regex="ok")
 
     def test_compat_with_concurrent_futures_exception(self):
         # It should be possible to use a loky process pool executor as a dropin
@@ -722,7 +726,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         # Ensure that loky.Future are compatible with concurrent.futures
         # (see #155)
         assert isinstance(f, concurrent.futures.Future)
-        (done, running) = concurrent.futures.wait([f], timeout=15)
+        _, running = concurrent.futures.wait([f], timeout=15)
         assert len(running) == 0
 
     thread_configurations = [
@@ -741,7 +745,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         else:
             # Break the shared executor before launching the threads:
             with pytest.raises(TerminatedWorkerError,
-                               match=filter_match(r"SIGSEGV")):
+                               match=filter_match("SIGSEGV")):
                 executor = get_reusable_executor(reuse=False)
                 executor.submit(return_instance, CrashAtPickle).result()
 
@@ -750,7 +754,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
             with warnings.catch_warnings():  # ignore resize warnings
                 warnings.simplefilter("always")
                 executor = get_reusable_executor(max_workers=max_workers)
-                for i in range(n_outer_steps):
+                for _ in range(n_outer_steps):
                     results = executor.map(
                         lambda x: x ** 2, range(n_inner_steps))
                     expected_result = [x ** 2 for x in range(n_inner_steps)]
@@ -766,7 +770,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         output_collector = []
         threads = [threading.Thread(
             target=helper_func, args=(output_collector, w),
-            name='test_thread_%02d_max_workers_%d' % (i, w))
+            name=f'test_thread_{i:02d}_max_workers_{w:d}')
             for i, w in enumerate(max_workers)]
 
         with warnings.catch_warnings(record=True):
