@@ -8,26 +8,14 @@ import threading
 import subprocess
 import contextlib
 from tempfile import mkstemp, mkdtemp, NamedTemporaryFile, _RandomNameSequence
-from loky.backend import resource_tracker
-if sys.version_info < (3, 4):
-    from loky.backend.semlock import SemLock as _SemLock
-    from loky.backend.semlock import sem_unlink
-else:
-    from _multiprocessing import SemLock as _SemLock
-    from _multiprocessing import sem_unlink
+from _multiprocessing import SemLock as _SemLock
+from _multiprocessing import sem_unlink
 
-try:
-    FileNotFoundError = FileNotFoundError
-except NameError:  # FileNotFoundError is Python 3-only
-    from loky.backend.semlock import FileNotFoundError
+from loky.backend import resource_tracker
 
 
 _rand_name = _RandomNameSequence()
 
-if sys.version_info[0] == 2:
-    class TimeoutError(OSError):
-        pass
-
 
 def resource_unlink(name, rtype):
     resource_tracker._CLEANUP_FUNCS[rtype](name)
@@ -36,9 +24,8 @@ def resource_unlink(name, rtype):
 def create_resource(rtype):
     if rtype == "folder":
         return mkdtemp(dir=os.getcwd())
-
     elif rtype == "semlock":
-        name = "test-loky-%i-%s" % (os.getpid(), next(_rand_name))
+        name = f"test-loky-{os.getpid()}-{next(_rand_name)}"
         _SemLock(1, 1, 1, name, False)
         return name
     elif rtype == "file":
@@ -46,7 +33,7 @@ def create_resource(rtype):
         tmpfile.close()
         return tmpfile.name
     else:
-        raise ValueError("Resource type %s not understood" % rtype)
+        raise ValueError(f"Resource type {rtype} not understood")
 
 
 def resource_exists(name, rtype):
@@ -60,7 +47,7 @@ def resource_exists(name, rtype):
         except OSError:
             return True
     else:
-        raise ValueError("Resource type %s not understood" % rtype)
+        raise ValueError(f"Resource type {rtype} not understood")
 
 
 @contextlib.contextmanager
@@ -70,13 +57,6 @@ def captured_output(stream_name):
     import io
     orig_stdout = getattr(sys, stream_name)
     s = io.StringIO()
-    if sys.version_info[:2] < (3, 3):
-        import types
-        s.wrt = s.write
-
-        def write(self, msg):
-            self.wrt(unicode(msg))
-        s.write = types.MethodType(write, s)
 
     setattr(sys, stream_name, s)
     try:
@@ -99,7 +79,7 @@ def captured_stderr():
 # Wrapper
 #
 
-class TimingWrapper(object):
+class TimingWrapper:
 
     def __init__(self, func):
         self.func = func
@@ -112,11 +92,15 @@ class TimingWrapper(object):
         finally:
             self.elapsed = time.time() - t
 
-    def assert_timing_almost_equal(self, delay):
-        assert round(self.elapsed - delay, 1) == 0
+    def assert_timing_lower_than(self, delay):
+        msg = (
+            f"expected duration lower than {delay:.3f}s, "
+            f"got {self.elapsed:.3f}s"
+        )
+        assert self.elapsed < delay, msg
 
     def assert_timing_almost_zero(self):
-        self.assert_timing_almost_equal(0.0)
+        self.assert_timing_lower_than(0.1)
 
 
 #
@@ -137,14 +121,13 @@ def check_subprocess_call(cmd, timeout=1, stdout_regex=None,
     stderr if stderr_regex is set.
     """
     if env is not None:
-        env_ = os.environ.copy()
-        env_.update(env)
-        env = env_
+        env = {**os.environ, **env}
+
     proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
-                            stderr=subprocess.PIPE, env=env)
+                            stderr=subprocess.PIPE, env=env, text=True)
 
     def kill_process():
-        warnings.warn("Timeout running {}".format(cmd))
+        warnings.warn(f"Timeout running {cmd}")
         proc.kill()
 
     timer = threading.Timer(timeout, kill_process)
@@ -152,30 +135,29 @@ def check_subprocess_call(cmd, timeout=1, stdout_regex=None,
         timer.start()
         stdout, stderr = proc.communicate()
 
-        if sys.version_info[0] >= 3:
-            stdout, stderr = stdout.decode(), stderr.decode()
         if proc.returncode == -9:
             message = (
-                'Subprocess timeout after {}s.\nStdout:\n{}\n'
-                'Stderr:\n{}').format(timeout, stdout, stderr)
+                f'Subprocess timeout after {timeout}s.\nStdout:\n{stdout}\n'
+                f'Stderr:\n{stderr}')
             raise TimeoutError(message)
         elif proc.returncode != 0:
             message = (
-                'Non-zero return code: {}.\nStdout:\n{}\n'
-                'Stderr:\n{}').format(
-                    proc.returncode, stdout, stderr)
+                f'Non-zero return code: {proc.returncode}.\nStdout:\n{stdout}'
+                f'\nStderr:\n{stderr}')
             raise ValueError(message)
 
         if (stdout_regex is not None and
                 not re.search(stdout_regex, stdout)):
             raise ValueError(
-                "Unexpected stdout: {!r} does not match:\n{!r}".format(
-                    stdout_regex, stdout))
+                f"Unexpected stdout: {stdout_regex!r} does not match:"
+                f"\n{stdout!r}"
+            )
         if (stderr_regex is not None and
                 not re.search(stderr_regex, stderr)):
             raise ValueError(
-                "Unexpected stderr: {!r} does not match:\n{!r}".format(
-                    stderr_regex, stderr))
+                f"Unexpected stderr: {stderr_regex!r} does not "
+                f"match:\n{stderr!r}"
+            )
 
         return stdout, stderr
 
@@ -227,19 +209,15 @@ def check_python_subprocess_call(code, stdout_regex=None):
     try:
         fid, filename = mkstemp(suffix="_joblib.py")
         os.close(fid)
-        with open(filename, mode='wb') as f:
-            f.write(code.encode('ascii'))
+        with open(filename, mode='w') as f:
+            f.write(code)
         cmd += [filename]
         check_subprocess_call(cmd, stdout_regex=stdout_regex, timeout=10)
     finally:
         os.unlink(filename)
 
 
-def filter_match(match, start_method=None):
+def filter_match(match):
     if sys.platform == "win32":
-        return
-
-    if start_method == "forkserver" and sys.version_info < (3, 7):
-        return "UNKNOWN"
-
+        return None
     return match
